@@ -172,11 +172,66 @@ class ServerRebuild(task_manager.Task):
         return client.nova_client.servers.rebuild(**self.args)
 
 
+class ServerGroupList(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.server_groups.list(**self.args)
+
+
+class ServerGroupGet(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.server_groups.get(**self.args)
+
+
+class ServerGroupCreate(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.server_groups.create(**self.args)
+
+
+class ServerGroupDelete(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.server_groups.delete(**self.args)
+
+
 class HypervisorList(task_manager.Task):
     def main(self, client):
         return client.nova_client.hypervisors.list(**self.args)
 
 
+class AggregateList(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.aggregates.list(**self.args)
+
+
+class AggregateCreate(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.aggregates.create(**self.args)
+
+
+class AggregateUpdate(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.aggregates.update(**self.args)
+
+
+class AggregateDelete(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.aggregates.delete(**self.args)
+
+
+class AggregateAddHost(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.aggregates.add_host(**self.args)
+
+
+class AggregateRemoveHost(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.aggregates.remove_host(**self.args)
+
+
+class AggregateSetMetadata(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.aggregates.set_metadata(**self.args)
+
+
 class KeypairList(task_manager.Task):
     def main(self, client):
         return client.nova_client.keypairs.list()
