@@ -14,11 +14,14 @@
 
 import contextlib
 import inspect
+import munch
 import netifaces
+import re
 import six
 import time
 
 from decorator import decorator
+from heatclient import exc as heat_exc
 from neutronclient.common import exceptions as neutron_exc
 
 from shade import _log
@@ -115,7 +118,7 @@ def _filter_list(data, name_or_id, filters):
     return filtered
 
 
-def _get_entity(func, name_or_id, filters):
+def _get_entity(func, name_or_id, filters, **kwargs):
     """Return a single entity from the list returned by a given method.
 
     :param callable func:
@@ -133,7 +136,7 @@ def _get_entity(func, name_or_id, filters):
     # object and just short-circuit return it.
     if hasattr(name_or_id, 'id'):
         return name_or_id
-    entities = func(name_or_id, filters)
+    entities = func(name_or_id, filters, **kwargs)
     if not entities:
         return None
     if len(entities) > 1:
@@ -191,6 +194,7 @@ def normalize_keystone_services(services):
             'description': service.get('description', None),
             'type': service_type,
             'service_type': service_type,
+            'enabled': service['enabled']
         }
         ret.append(new_service)
     return meta.obj_list_to_dict(ret)
@@ -309,15 +313,21 @@ def normalize_neutron_floating_ips(ips):
         ]
 
     """
-    ret = [dict(
-        id=ip['id'],
-        fixed_ip_address=ip.get('fixed_ip_address'),
-        floating_ip_address=ip['floating_ip_address'],
-        network=ip['floating_network_id'],
-        attached=(ip.get('port_id') is not None and
-                  ip.get('port_id') != ''),
-        status=ip['status']
-    ) for ip in ips]
+    ret = []
+    for ip in ips:
+        network_id = ip.get('floating_network_id', ip.get('network'))
+        ret.append(dict(
+            id=ip['id'],
+            fixed_ip_address=ip.get('fixed_ip_address'),
+            floating_ip_address=ip['floating_ip_address'],
+            network=network_id,
+            floating_network_id=network_id,
+            port_id=ip.get('port_id'),
+            router_id=ip.get('router_id'),
+            attached=(ip.get('port_id') is not None and
+                      ip.get('port_id') != ''),
+            status=ip['status'],
+        ))
     return meta.obj_list_to_dict(ret)
 
 
@@ -436,7 +446,7 @@ def normalize_role_assignments(assignments):
     """
     new_assignments = []
     for assignment in assignments:
-        new_val = {'id': assignment['role']['id']}
+        new_val = munch.Munch({'id': assignment['role']['id']})
         for scope in ('project', 'domain'):
             if scope in assignment['scope']:
                 new_val[scope] = assignment['scope'][scope]['id']
@@ -447,6 +457,44 @@ def normalize_role_assignments(assignments):
     return new_assignments
 
 
+def normalize_roles(roles):
+    """Normalize Identity roles."""
+    ret = [
+        dict(
+            id=role.get('id'),
+            name=role.get('name'),
+        ) for role in roles
+    ]
+    return meta.obj_list_to_dict(ret)
+
+
+def normalize_stacks(stacks):
+    """ Normalize Stack Object """
+    for stack in stacks:
+        stack['name'] = stack['stack_name']
+    return stacks
+
+
+def normalize_flavors(flavors):
+    """ Normalize a list of flavor objects """
+    for flavor in flavors:
+        flavor.pop('links', None)
+        flavor.pop('NAME_ATTR', None)
+        flavor.pop('HUMAN_ID', None)
+        flavor.pop('human_id', None)
+        if 'extra_specs' not in flavor:
+            flavor['extra_specs'] = {}
+        ephemeral = flavor.pop('OS-FLV-EXT-DATA:ephemeral', 0)
+        is_public = flavor.pop('os-flavor-access:is_public', True)
+        # Make sure both the extension version and a sane version are present
+        flavor['OS-FLV-EXT-DATA:ephemeral'] = ephemeral
+        flavor['ephemeral'] = ephemeral
+        flavor['os-flavor-access:is_public'] = is_public
+        flavor['is_public'] = is_public
+
+    return flavors
+
+
 def valid_kwargs(*valid_args):
     # This decorator checks if argument passed as **kwargs to a function are
     # present in valid_args.
@@ -493,6 +541,18 @@ def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
     return _inner_cache_on_arguments
 
 
+@contextlib.contextmanager
+def heat_exceptions(error_message):
+    try:
+        yield
+    except heat_exc.NotFound as e:
+        raise exc.OpenStackCloudResourceNotFound(
+            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
+    except Exception as e:
+        raise exc.OpenStackCloudException(
+            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
+
+
 @contextlib.contextmanager
 def neutron_exceptions(error_message):
     try:
@@ -535,3 +595,167 @@ def shade_exceptions(error_message=None):
         if error_message is None:
             error_message = str(e)
         raise exc.OpenStackCloudException(error_message)
+
+
+def safe_dict_min(key, data):
+    """Safely find the minimum for a given key in a list of dict objects.
+
+    This will find the minimum integer value for specific dictionary key
+    across a list of dictionaries. The values for the given key MUST be
+    integers, or string representations of an integer.
+
+    The dictionary key does not have to be present in all (or any)
+    of the elements/dicts within the data set.
+
+    :param string key: The dictionary key to search for the minimum value.
+    :param list data: List of dicts to use for the data set.
+
+    :returns: None if the field was not not found in any elements, or
+        the minimum value for the field otherwise.
+    """
+    min_value = None
+    for d in data:
+        if (key in d) and (d[key] is not None):
+            try:
+                val = int(d[key])
+            except ValueError:
+                raise exc.OpenStackCloudException(
+                    "Search for minimum value failed. "
+                    "Value for {key} is not an integer: {value}".format(
+                        key=key, value=d[key])
+                )
+            if (min_value is None) or (val < min_value):
+                min_value = val
+    return min_value
+
+
+def safe_dict_max(key, data):
+    """Safely find the maximum for a given key in a list of dict objects.
+
+    This will find the maximum integer value for specific dictionary key
+    across a list of dictionaries. The values for the given key MUST be
+    integers, or string representations of an integer.
+
+    The dictionary key does not have to be present in all (or any)
+    of the elements/dicts within the data set.
+
+    :param string key: The dictionary key to search for the maximum value.
+    :param list data: List of dicts to use for the data set.
+
+    :returns: None if the field was not not found in any elements, or
+        the maximum value for the field otherwise.
+    """
+    max_value = None
+    for d in data:
+        if (key in d) and (d[key] is not None):
+            try:
+                val = int(d[key])
+            except ValueError:
+                raise exc.OpenStackCloudException(
+                    "Search for maximum value failed. "
+                    "Value for {key} is not an integer: {value}".format(
+                        key=key, value=d[key])
+                )
+            if (max_value is None) or (val > max_value):
+                max_value = val
+    return max_value
+
+
+def parse_range(value):
+    """Parse a numerical range string.
+
+    Breakdown a range expression into its operater and numerical parts.
+    This expression must be a string. Valid values must be an integer string,
+    optionally preceeded by one of the following operators::
+
+        - "<"  : Less than
+        - ">"  : Greater than
+        - "<=" : Less than or equal to
+        - ">=" : Greater than or equal to
+
+    Some examples of valid values and function return values::
+
+        - "1024"  : returns (None, 1024)
+        - "<5"    : returns ("<", 5)
+        - ">=100" : returns (">=", 100)
+
+    :param string value: The range expression to be parsed.
+
+    :returns: A tuple with the operator string (or None if no operator
+        was given) and the integer value. None is returned if parsing failed.
+    """
+    if value is None:
+        return None
+
+    range_exp = re.match('(<|>|<=|>=){0,1}(\d+)$', value)
+    if range_exp is None:
+        return None
+
+    op = range_exp.group(1)
+    num = int(range_exp.group(2))
+    return (op, num)
+
+
+def range_filter(data, key, range_exp):
+    """Filter a list by a single range expression.
+
+    :param list data: List of dictionaries to be searched.
+    :param string key: Key name to search within the data set.
+    :param string range_exp: The expression describing the range of values.
+
+    :returns: A list subset of the original data set.
+    :raises: OpenStackCloudException on invalid range expressions.
+    """
+    filtered = []
+    range_exp = str(range_exp).upper()
+
+    if range_exp == "MIN":
+        key_min = safe_dict_min(key, data)
+        if key_min is None:
+            return []
+        for d in data:
+            if int(d[key]) == key_min:
+                filtered.append(d)
+        return filtered
+    elif range_exp == "MAX":
+        key_max = safe_dict_max(key, data)
+        if key_max is None:
+            return []
+        for d in data:
+            if int(d[key]) == key_max:
+                filtered.append(d)
+        return filtered
+
+    # Not looking for a min or max, so a range or exact value must
+    # have been supplied.
+    val_range = parse_range(range_exp)
+
+    # If parsing the range fails, it must be a bad value.
+    if val_range is None:
+        raise exc.OpenStackCloudException(
+            "Invalid range value: {value}".format(value=range_exp))
+
+    op = val_range[0]
+    if op:
+        # Range matching
+        for d in data:
+            d_val = int(d[key])
+            if op == '<':
+                if d_val < val_range[1]:
+                    filtered.append(d)
+            elif op == '>':
+                if d_val > val_range[1]:
+                    filtered.append(d)
+            elif op == '<=':
+                if d_val <= val_range[1]:
+                    filtered.append(d)
+            elif op == '>=':
+                if d_val >= val_range[1]:
+                    filtered.append(d)
+        return filtered
+    else:
+        # Exact number match
+        for d in data:
+            if int(d[key]) == val_range[1]:
+                filtered.append(d)
+        return filtered
