@@ -114,6 +114,13 @@ class FakeServer(object):
         self.interface_ip = interface_ip
 
 
+class FakeServerGroup(object):
+    def __init__(self, id, name, policies):
+        self.id = id
+        self.name = name
+        self.policies = policies
+
+
 class FakeService(object):
     def __init__(self, id, name, type, service_type, description='',
                  enabled=True):
@@ -272,3 +279,17 @@ class FakeRecordset(object):
         self.description = description
         self.ttl = ttl
         self.records = records
+
+
+class FakeAggregate(object):
+    def __init__(self, id, name, availability_zone=None, metadata=None,
+                 hosts=None):
+        self.id = id
+        self.name = name
+        self.availability_zone = availability_zone
+        if not metadata:
+            metadata = {}
+        self.metadata = metadata
+        if not hosts:
+            hosts = []
+        self.hosts = hosts
