@@ -242,9 +242,22 @@ class TestFloatingIP(base.TestCase):
         mock_has_service.side_effect = has_service_side_effect
         mock_nova_client.floating_ips.list.return_value = self.floating_ips
 
-        ip = self.client._add_ip_from_pool(
+        server = self.client._add_ip_from_pool(
             server=self.fake_server,
             network='nova',
             fixed_address='192.0.2.129')
 
-        self.assertEqual('203.0.113.1', ip['floating_ip_address'])
+        self.assertEqual(server, self.fake_server)
+
+    @patch.object(OpenStackCloud, 'delete_floating_ip')
+    @patch.object(OpenStackCloud, 'list_floating_ips')
+    @patch.object(OpenStackCloud, '_use_neutron_floating')
+    def test_cleanup_floating_ips(
+            self, mock_use_neutron_floating, mock_list_floating_ips,
+            mock_delete_floating_ip):
+        mock_use_neutron_floating.return_value = False
+
+        self.client.delete_unattached_floating_ips()
+
+        mock_delete_floating_ip.assert_not_called()
+        mock_list_floating_ips.assert_not_called()
